@@ -20,28 +20,27 @@
 # Germano Cavalcante (mano-wii), Florian Meyer (testscreenings), Brendon Murphy (meta-androcto) and Bart Crouch
 
 
-# To install this addon you need to install 'protobuf' and 'grpcio' to the blender python installation.
+# To install this addon you need to install 'toxicblend', 'protobuf' and 'grpcio' to the blender python installation.
 # This is an example of how this can be done: Open the script tab in blender and run this:
+#
 # """
 # import subprocess
-#  import sys
-#  import os
-#  import site
+# import sys
+# import os
+# import site
 #
-#  print("python executable:", sys.executable)
+# print("this is the path to the python executable: ", sys.executable)
 #
-#  # upgrade pip
-#  subprocess.call([sys.executable, "-m", "ensurepip"])
-#  subprocess.call([sys.executable, "-m", "pip", "install", "--upgrade", "pip"])
-#
-#  # install required packages
-#  subprocess.call([sys.executable, "-m", "pip", "install", "protobuf"])
-#  subprocess.call([sys.executable, "-m", "pip", "install", "grpcio"])
+# # upgrade pip
+# subprocess.call([sys.executable, "-m", "ensurepip"])
+# subprocess.call([sys.executable, "-m", "pip", "install", "--upgrade", "pip"])
+
+# # install the pip package toxicblend + the dependencies protobuf and grpcio
+# subprocess.call([sys.executable, "-m", "pip", "install", "--upgrade", "toxicblend"])
 # """
 #
-# You will also have to copy the toxicblend_pb2.py and toxicblend_pb2_grpc.py files to the
-# site packages of the same python environment.
-# See install_as_blender_addon.md for examples
+# Then you need to install this file as a blender addon as usual.
+# See install_as_blender_addon.md details
 
 
 from __future__ import print_function
@@ -80,7 +79,7 @@ bl_info = {
     "category": "Mesh",
 }
 
-SOCKET = 'localhost:50069'
+SERVER_URL = 'localhost:50069'
 
 
 class ToxicblendException(Exception):
@@ -137,14 +136,6 @@ def terminate():
         bmesh.update_edit_mesh(obj.data, loop_triangles=True, destructive=True)
 
 
-# make an edge key object
-def make_key(v0, v1):
-    if v0 < v1:
-        return v0, v1
-    else:
-        return v1, v0
-
-
 IDENTITY4x4 = mathutils.Matrix.Identity(4)
 
 
@@ -176,46 +167,51 @@ def build_pb_model(bpy_object, bm, pb_model):
         for v in f.verts:
             pb_face.vertices.append(v.index)
             if prev_vertex:
-                sent_edges.add(make_key(prev_vertex, v.index))
+                sent_edges.add(make_edge_key(prev_vertex, v.index))
 
             prev_vertex = v.index
             if prev_vertex and len(f.verts) > 0:
                 first_vertex = f.verts[0].index
-                sent_edges.add(make_key(first_vertex, prev_vertex))
+                sent_edges.add(make_edge_key(first_vertex, prev_vertex))
 
     for e in bm.edges:
         f = e.verts[0].index
         t = e.verts[1].index
-        key = make_key(f, t)
+        key = make_edge_key(f, t)
         if key not in sent_edges:
             pb_face = pb_model.faces.add()
             pb_face.vertices.append(f)
             pb_face.vertices.append(t)
 
 
-def get_pydata(pb_model, only_edges=False):
-    """Parse the received proto buffer data into something useful"""
+def get_pydata(pb_model, only_edges=False, packed_faces=False):
+    """Convert the received proto buffer data into something useful"""
     rv_vertices = [(v.x, v.y, v.z) for v in pb_model.vertices]
     rv_edges = []
     rv_faces = []
-    for f in pb_model.faces:
-        vertices = []
-        for v in f.vertices:
-            if 0 <= v < len(rv_vertices):
-                vertices.append(v)
-            else:
-                print("Vertex %d is unknown -> ignored" % (v,))
-        no_vertices = len(vertices)
-        if no_vertices > 1:
-            if only_edges or no_vertices == 2:
-                if vertices[0] != vertices[1]:
-                    rv_edges.append(vertices)
+    if packed_faces:
+        for face in pb_model.faces:
+            for i in range(0, len(face.vertices), 3):
+                rv_faces.append([face.vertices[i], face.vertices[i + 1], face.vertices[i + 2]])
+    else:
+        for f in pb_model.faces:
+            vertices = []
+            for v in f.vertices:
+                if 0 <= v < len(rv_vertices):
+                    vertices.append(v)
                 else:
-                    print("FIXME: was asked to make an edge between two identical vertices: %s and %s" % (
-                        vertices[0], vertices[1]))
-                # rvFaces.append(vertices)
-            else:
-                rv_faces.append(vertices)
+                    print("Vertex %d is unknown -> ignored" % (v,))
+            no_vertices = len(vertices)
+            if no_vertices > 1:
+                if only_edges or no_vertices == 2:
+                    if vertices[0] != vertices[1]:
+                        rv_edges.append(vertices)
+                    else:
+                        print("FIXME: was asked to make an edge between two identical vertices: %s and %s" % (
+                            vertices[0], vertices[1]))
+                else:
+                    # print("Adding face:", vertices)
+                    rv_faces.append(vertices)
 
     mat = IDENTITY4x4.copy()
     if pb_model.HasField("worldOrientation"):
@@ -235,17 +231,20 @@ def handle_response(pb_message):
 
 def handle_received_object(active_object, pb_message, remove_doubles_threshold=None, set_origin_to_cursor=False):
     only_edges = False
+    packed_faces = False
     for option in pb_message.options:
         if option.key == "ERROR":
             raise ToxicblendException(str(option.value))
         if option.key == "ONLY_EDGES" and option.value == "True":
             only_edges = True
+        if option.key == "PACKED_FACES" and option.value == "True":
+            packed_faces = True
 
     # active_object.select = False
 
     if len(pb_message.models) == 1:
         pb_model = pb_message.models[0]
-        (vertices, edges, faces, matrix) = get_pydata(pb_model, only_edges)
+        (vertices, edges, faces, matrix) = get_pydata(pb_model, only_edges, packed_faces)
         if len(faces) > 0 or len(edges) > 0:
             new_mesh = bpy.data.meshes.new(pb_model.name)
             # ob = bpy.data.objects.new(pb_model.name, new_mesh)
@@ -284,6 +283,58 @@ def handle_received_object(active_object, pb_message, remove_doubles_threshold=N
                 bpy.ops.object.origin_set(type='ORIGIN_CURSOR')
 
 
+def angle_between_edges(p0, p1, p2):
+    """ angle between the two vectors defined as p0->p1 and p1->p2
+    return value in degrees """
+    v1 = p1 - p0
+    v2 = p2 - p1
+
+    v1mag = math.sqrt(v1.x * v1.x + v1.y * v1.y + v1.z * v1.z)
+    if v1mag == 0.0:
+        return 0.0
+
+    v1norm = [v1.x / v1mag, v1.y / v1mag, v1.z / v1mag]
+    v2mag = math.sqrt(v2.x * v2.x + v2.y * v2.y + v2.z * v2.z)
+    if v2mag == 0.0:
+        return 0.0
+
+    v2norm = [v2.x / v2mag, v2.y / v2mag, v2.z / v2mag]
+    res = v1norm[0] * v2norm[0] + v1norm[1] * v2norm[1] + v1norm[2] * v2norm[2]
+    angle = math.degrees(math.acos(res))
+    return angle
+
+
+# make an edge key object
+def make_edge_key(v0, v1):
+    if v0 < v1:
+        return v0, v1
+    else:
+        return v1, v0
+
+
+def append_value(dict_obj, key, value):
+    # Check if key exist in dict or not
+    if key in dict_obj:
+        # Key exist in dict.
+        # Check if type of value of key is list or not
+        if not isinstance(dict_obj[key], list):
+            # If type is not list then make it list
+            dict_obj[key] = [dict_obj[key]]
+        # Append the value in list
+        dict_obj[key].append(value)
+    else:
+        # As key is not in dict,
+        # so, add key-value pair
+        dict_obj[key] = [value]
+
+
+def get_or_empty_list(dict_obj, key):
+    if key in dict_obj:
+        return dict_obj[key]
+    else:
+        return []
+
+
 # ########################################
 # ##### Operators ########################
 # ########################################
@@ -391,6 +442,124 @@ class ToxicBlend_SelectEndVertices(Operator):
         return {'FINISHED'}
 
 
+class ToxicBlend_SelectCollinearEdges(Operator):
+    """Selects edges that are connected to the selected edges, but limit by an angle constraint (offline plugin)"""
+    bl_idname = "mesh.toxicblend_meshtools_select_collinear_edges"
+    bl_label = "Select collinear edges"
+    bl_description = "Selects edges that are connected to the selected edges, but limit by an angle constraint (offline plugin)"
+    bl_options = {'REGISTER', 'UNDO'}  # enable undo for the operator.
+
+    angle: FloatProperty(
+        name="Angle selection constraint",
+        description="selects edges with a relative angle (compared to an already selected edge) smaller than this.",
+        default=math.radians(12.0),
+        min=math.radians(0.0),
+        max=math.radians(180.0),
+        precision=6,
+        subtype='ANGLE'
+    )
+
+    @classmethod
+    def poll(cls, context):
+        return context.active_object is not None
+
+    def execute(self, context):
+
+        # Get the active mesh
+        obj = bpy.context.edit_object
+        me = obj.data
+
+        # Get a BMesh representation
+        bm = bmesh.from_edit_mesh(me)
+        bm.verts.ensure_lookup_table()
+        bm.edges.ensure_lookup_table()
+        bm.faces.ensure_lookup_table()
+
+        angle_criteria = math.degrees(self.angle)
+
+        # angle_between_edges(p0,p1,p2)
+        # make_edge_key(v0, v1):
+        # append_value(dict_obj, key, value):
+
+        vertex_dict = dict()  # key by vertex.index to [edges]
+        already_selected = set()  # key by edge.index
+        work_queue = set()  # edges
+
+        for e in bm.edges:
+            append_value(vertex_dict, e.verts[0].index, e)
+            append_value(vertex_dict, e.verts[1].index, e)
+            if e.select:
+                # already_selected.add(e.index)
+                work_queue.add(e)
+
+        # print("len(vertex_dict)", len(vertex_dict))
+        # print("len(already_selected)", len(already_selected))
+        # print("len(work_queue)", len(work_queue))
+
+        while len(work_queue) > 0:
+            e = work_queue.pop()
+            if e.index in already_selected:
+                continue
+
+            from_v = e.verts[0].index
+            to_v = e.verts[1].index
+
+            for candidate_e in get_or_empty_list(vertex_dict, to_v):
+                if candidate_e.select or candidate_e.index == e.index:
+                    continue
+
+                # print("from_v:", from_v, " is connected to edge :", candidate_e.index)
+                if to_v == candidate_e.verts[0].index:
+                    angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
+                                                bm.verts[candidate_e.verts[1].index].co)
+                    # print("from_v<->to_v<->candidate_e.verts[1] angle:", angle)
+                    if angle <= angle_criteria:
+                        # print("appending!", candidate_e)
+                        work_queue.add(candidate_e)
+
+                elif to_v == candidate_e.verts[1].index:
+                    angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
+                                                bm.verts[candidate_e.verts[0].index].co)
+                    # print("from_v<->to_v<->candidate_e.verts[0] angle:",angle)
+                    if angle <= angle_criteria:
+                        # print("appending!", candidate_e)
+                        work_queue.add(candidate_e)
+
+            # Do the search again, in the other direction
+            from_v = e.verts[1].index
+            to_v = e.verts[0].index
+
+            for candidate_e in get_or_empty_list(vertex_dict, to_v):
+                if candidate_e.select or candidate_e.index == e.index:
+                    continue
+
+                # print("from_v:", from_v, " is connected to edge :", candidate_e.index)
+                if to_v == candidate_e.verts[0].index:
+                    angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
+                                                bm.verts[candidate_e.verts[1].index].co)
+                    # print("from_v<->to_v<->candidate_e.verts[1] angle:", angle)
+                    if angle <= angle_criteria:
+                        # print("appending!", candidate_e)
+                        work_queue.add(candidate_e)
+
+                elif to_v == candidate_e.verts[1].index:
+                    angle = angle_between_edges(bm.verts[from_v].co, bm.verts[to_v].co,
+                                                bm.verts[candidate_e.verts[0].index].co)
+                    # print("from_v<->to_v<->candidate_e.verts[0] angle:", angle)
+                    if angle <= angle_criteria:
+                        # print("appending!", candidate_e)
+                        work_queue.add(candidate_e)
+
+            e.select = True
+            # only mark edges as already_selected if they've been through the loop once
+            already_selected.add(e.index)
+
+        # Show the updates in the viewport
+        bmesh.update_edit_mesh(me, False)
+
+        return {'FINISHED'}
+
+
 class ToxicBlend_SelectIntersectionVertices(Operator):
     """Selects all vertices that are connected to more than two other vertices (offline plugin)"""
     bl_idname = "mesh.toxicblend_meshtools_select_intersection_vertices"
@@ -453,12 +622,9 @@ class Toxicblend_2D_Outline(Operator):
         active_object, active_mesh = initialise()
         settings_write(self)
         try:
-            with grpc.insecure_channel(SOCKET) as channel:
+            with grpc.insecure_channel(SERVER_URL) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
                 command = toxicblend_pb2.Command(command='2d_outline')
-                opt = command.options.add()
-                opt.key = "a key"
-                opt.value = "a value"
                 build_pb_model(active_object, active_mesh, command.models.add())
 
                 response = stub.execute(command)
@@ -523,7 +689,7 @@ class Toxicblend_Voronoi(Operator):
         active_object, active_mesh = initialise()
         settings_write(self)
         try:
-            with grpc.insecure_channel(SOCKET) as channel:
+            with grpc.insecure_channel(SERVER_URL) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
                 command = toxicblend_pb2.Command(command='voronoi')
 
@@ -580,7 +746,7 @@ class Toxicblend_Knife_Intersect(Operator):
         active_object, active_mesh = initialise()
         settings_write(self)
         try:
-            with grpc.insecure_channel(SOCKET) as channel:
+            with grpc.insecure_channel(SERVER_URL) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
                 command = toxicblend_pb2.Command(command='knife_intersect')
                 build_pb_model(active_object, active_mesh, command.models.add())
@@ -663,7 +829,7 @@ class Toxicblend_Centerline(Operator):
         active_object, active_mesh = initialise()
         settings_write(self)
         try:
-            with grpc.insecure_channel(SOCKET) as channel:
+            with grpc.insecure_channel(SERVER_URL) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
                 command = toxicblend_pb2.Command(command='centerline')
                 build_pb_model(active_object, active_mesh, command.models.add())
@@ -702,18 +868,12 @@ class Toxicblend_Centerline(Operator):
 
 
 # Centerline operator
-class Toxicblend_Centerline_Mesh(Operator):
-    bl_idname = "mesh.toxicblend_meshtools_centerline_mesh"
-    bl_label = "Centerline Mesh"
-    bl_description = "Calculate centerline, the geometry must be flat and on a plane intersecting origin. This version of centerline tries to keep as many edges as possible. Intended for mesh generation by blender. (press F when done)"
+class Toxicblend_Voronoi_Mesh(Operator):
+    bl_idname = "mesh.toxicblend_meshtools_voronoi_mesh"
+    bl_label = "Voronoi Mesh"
+    bl_description = "Calculate voronoi diagram and add mesh, the geometry must be flat and on a plane intersecting origin."
     bl_options = {'REGISTER', 'UNDO'}
 
-    remove_internals: BoolProperty(
-        name="Remove internal edges",
-        description="Remove edges internal to islands in the geometry",
-        default=True
-    )
-
     distance: FloatProperty(
         name="Distance",
         description="Discrete distance as a percentage of the AABB",
@@ -724,12 +884,6 @@ class Toxicblend_Centerline_Mesh(Operator):
         subtype='PERCENTAGE'
     )
 
-    simplify: BoolProperty(
-        name="Simplify line strings",
-        description="Simplify voronoi edges connected as in a line string. The 'distance' property is used.",
-        default=True
-    )
-
     @classmethod
     def poll(cls, context):
         ob = context.active_object
@@ -745,13 +899,11 @@ class Toxicblend_Centerline_Mesh(Operator):
         active_object, active_mesh = initialise()
         settings_write(self)
         try:
-            with grpc.insecure_channel(SOCKET) as channel:
+            with grpc.insecure_channel(SERVER_URL) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
-                command = toxicblend_pb2.Command(command='centerline_mesh')
+                command = toxicblend_pb2.Command(command='voronoi_mesh')
                 build_pb_model(active_object, active_mesh, command.models.add())
-                opt = command.options.add()
-                opt.key = "REMOVE_INTERNALS"
-                opt.value = str(self.remove_internals).lower()
+
                 opt = command.options.add()
                 opt.key = "DISTANCE"
                 opt.value = str(self.distance)
@@ -773,43 +925,32 @@ class Toxicblend_Centerline_Mesh(Operator):
             self.report({'ERROR'}, str(e))
             return {'CANCELLED'}
 
-# Centerline operator
+
+# Voxel operator
 class Toxicblend_Voxel(Operator):
     bl_idname = "mesh.toxicblend_meshtools_voxel"
     bl_label = "Voxel"
     bl_description = "Calculate voxel tubes around edges"
     bl_options = {'REGISTER', 'UNDO'}
 
-    angle: FloatProperty(
-        name="Angle",
-        description="Edge rejection angle, edges with edge-to-segment angles larger than this will be rejected",
-        default=math.radians(50.0),
-        min=math.radians(0.000001),
-        max=math.radians(89.999999),
-        precision=6,
-        subtype='ANGLE',
-    )
-
-    remove_internals: BoolProperty(
-        name="Remove internal edges",
-        description="Remove edges internal to islands in the geometry",
-        default=True
-    )
-
-    distance: FloatProperty(
-        name="Distance",
-        description="Discrete distance as a percentage of the AABB",
-        default=0.005,
-        min=0.0001,
+    radius: FloatProperty(
+        name="Radius",
+        description="Voxel tube radius as a percentage of the total AABB",
+        default=1.0,
+        min=0.01,
         max=4.9999,
         precision=6,
         subtype='PERCENTAGE'
     )
 
-    simplify: BoolProperty(
-        name="Simplify line strings",
-        description="Simplify voronoi edges connected as in a line string. The 'distance' property is used.",
-        default=True
+    divisions: FloatProperty(
+        name="Voxel Divisions",
+        description="The larges scale of the AABB will be divided up into this number of voxels",
+        default=100.0,
+        min=50,
+        max=200,
+        precision=1,
+        subtype='FACTOR'
     )
 
     @classmethod
@@ -827,22 +968,20 @@ class Toxicblend_Voxel(Operator):
         active_object, active_mesh = initialise()
         settings_write(self)
         try:
-            with grpc.insecure_channel(SOCKET) as channel:
+            channel_opt = [('grpc.max_send_message_length', 512 * 1024 * 1024),
+                           ('grpc.max_receive_message_length', 512 * 1024 * 1024)]
+            with grpc.insecure_channel(SERVER_URL, options=channel_opt) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
                 command = toxicblend_pb2.Command(command='voxel')
                 build_pb_model(active_object, active_mesh, command.models.add())
+
                 opt = command.options.add()
-                opt.key = "ANGLE"
-                opt.value = str(math.degrees(self.angle))
+                opt.key = "RADIUS"
+                opt.value = str(self.radius)
+
                 opt = command.options.add()
-                opt.key = "REMOVE_INTERNALS"
-                opt.value = str(self.remove_internals).lower()
-                opt = command.options.add()
-                opt.key = "DISTANCE"
-                opt.value = str(self.distance)
-                opt = command.options.add()
-                opt.key = "SIMPLIFY"
-                opt.value = str(self.simplify).lower()
+                opt.key = "DIVISIONS"
+                opt.value = str(self.divisions)
 
                 response = stub.execute(command)
                 handle_response(response)
@@ -861,6 +1000,7 @@ class Toxicblend_Voxel(Operator):
             self.report({'ERROR'}, str(e))
             return {'CANCELLED'}
 
+
 # 2d_outline operator
 class Toxicblend_Simplify(Operator):
     bl_idname = "mesh.toxicblend_meshtools_simplify"
@@ -893,7 +1033,7 @@ class Toxicblend_Simplify(Operator):
         active_object, active_mesh = initialise()
         settings_write(self)
         try:
-            with grpc.insecure_channel(SOCKET) as channel:
+            with grpc.insecure_channel(SERVER_URL) as channel:
                 stub = toxicblend_pb2_grpc.ToxicBlendServiceStub(channel)
                 command = toxicblend_pb2.Command(command='simplify')
                 opt = command.options.add()
@@ -933,10 +1073,11 @@ class VIEW3D_MT_edit_mesh_toxicblend_meshtools(Menu):
         layout.operator("mesh.toxicblend_meshtools_2d_outline")
         layout.operator("mesh.toxicblend_meshtools_knife_intersect")
         layout.operator("mesh.toxicblend_meshtools_centerline")
-        layout.operator("mesh.toxicblend_meshtools_centerline_mesh")
+        layout.operator("mesh.toxicblend_meshtools_voronoi_mesh")
         layout.operator("mesh.toxicblend_meshtools_voronoi")
-        #layout.operator("mesh.toxicblend_meshtools_voxel")
+        layout.operator("mesh.toxicblend_meshtools_voxel")
         layout.operator("mesh.toxicblend_meshtools_select_end_vertices")
+        layout.operator("mesh.toxicblend_meshtools_select_collinear_edges")
         layout.operator("mesh.toxicblend_meshtools_select_intersection_vertices")
         layout.operator("mesh.toxicblend_meshtools_debug_object")
 
@@ -1008,19 +1149,13 @@ class TB_MeshToolsProps(PropertyGroup):
         default=True
     )
 
-    centerline_mesh_remove_internals: BoolProperty(
-        name="Remove internal edges",
-        description="Remove edges internal to islands in the geometry",
-        default=True
-    )
-
     centerline_weld: BoolProperty(
         name="Weld the centerline to outline",
         description="Centerline and outline will share vertices if they intersect",
         default=True
     )
 
-    centerline_mesh_distance: FloatProperty(
+    voronoi_mesh_distance: FloatProperty(
         name="Distance",
         description="Discrete distance as a percentage of the AABB",
         default=0.005,
@@ -1030,12 +1165,6 @@ class TB_MeshToolsProps(PropertyGroup):
         subtype='PERCENTAGE'
     )
 
-    centerline_mesh_simplify: BoolProperty(
-        name="Simplify line strings",
-        description="Simplify voronoi edges connected as in a line string. The 'distance' property is used.",
-        default=True
-    )
-
     voronoi_remove_externals: BoolProperty(
         name="Remove external edges",
         description="Remove edges connected or indirectly connected to 'infinite' edges. Edges inside input geometry are always considered 'internal'",
@@ -1058,6 +1187,36 @@ class TB_MeshToolsProps(PropertyGroup):
         default=False
     )
 
+    select_collinear_edges_angle: FloatProperty(
+        name="Angle selection constraint",
+        description="selects edges with an angle smaller than this.",
+        default=math.radians(12.0),
+        min=math.radians(0.0),
+        max=math.radians(180.0),
+        precision=6,
+        subtype='ANGLE'
+    )
+
+    voxel_radius: FloatProperty(
+        name="Radius",
+        description="Voxel tube radius as a percentage of the total AABB",
+        default=1.0,
+        min=0.01,
+        max=4.9999,
+        precision=6,
+        subtype='PERCENTAGE'
+    )
+
+    voxel_divisions: FloatProperty(
+        name="Voxel Divisions",
+        description="The larges scale of the AABB will be divided up into this number of voxels",
+        default=100.0,
+        min=50,
+        max=200,
+        precision=1,
+        subtype='FACTOR'
+    )
+
 
 # draw function for integration in menus
 def menu_func(self, context):
@@ -1073,11 +1232,12 @@ classes = (
     Toxicblend_2D_Outline,
     Toxicblend_Knife_Intersect,
     Toxicblend_Centerline,
-    Toxicblend_Centerline_Mesh,
+    Toxicblend_Voronoi_Mesh,
     Toxicblend_Voronoi,
-    #Toxicblend_Voxel,
+    Toxicblend_Voxel,
     ToxicBlend_SelectEndVertices,
     ToxicBlend_SelectIntersectionVertices,
+    ToxicBlend_SelectCollinearEdges,
     ToxicBlend_debug_object,
 )
 
